@@ -65,12 +65,14 @@ var PlaceEntry = GObject.registerClass({
             return;
 
         if (p) {
-            if (p.name) {
+            if (p.name)
                 this._placeText = p.name;
-            } else
-                this._placeText = p.location.latitude + ', ' + p.location.longitude;
-        } else
+            else
+                this._placeText = '%.5f, %.5f'.format(p.location.latitude,
+                                                      p.location.longitude);
+        } else {
             this._placeText = '';
+        }
 
         this.text = this._placeText;
 
@@ -259,7 +261,7 @@ var PlaceEntry = GObject.registerClass({
             this._updateResults(places);
 
             // cache results for later
-            this._cache[this._previousSearch] = places;
+            this._cache[this.text] = places;
 
             // if search input has been updated, trigger a refresh
             if (this.text !== this._previousSearch)
